@@ -1,16 +1,13 @@
+# -*- coding: utf-8 -*-
 import re
+from importlib import import_module
+
 from django.conf import settings
 from django.utils.safestring import mark_safe
-from django.utils.importlib import import_module
+from django.contrib.auth import get_user_model
 from django.core.paginator import Paginator, EmptyPage
 
-try:
-    # Django 1.5 check
-    from django.contrib.auth import get_user_model
-except ImportError:
-    from django.contrib.auth.models import User
-else:
-    User = get_user_model()
+User = get_user_model()
 
 
 def get_redir_path(request=None):
@@ -25,7 +22,7 @@ def get_redir_path(request=None):
     return nextval or getattr(
         settings,
         'IMPERSONATE_REDIRECT_URL',
-        getattr(settings, 'LOGIN_REDIRECT_URL', '/'),
+        getattr(settings, 'LOGIN_REDIRECT_URL', u'/'),
     )
 
 
@@ -38,8 +35,8 @@ def get_redir_arg(request):
     if redirect_field_name:
         nextval = request.GET.get(redirect_field_name, None)
         if nextval:
-            return '?{0}={1}'.format(redirect_field_name, nextval)
-    return ''
+            return u'?{0}={1}'.format(redirect_field_name, nextval)
+    return u''
 
 
 def get_redir_field(request):
@@ -52,12 +49,12 @@ def get_redir_field(request):
         nextval = request.GET.get(redirect_field_name, None)
         if nextval:
             return mark_safe(
-                '<input type="hidden" name="{0}" value="{1}"/>'.format(
+                u'<input type="hidden" name="{0}" value="{1}"/>'.format(
                     redirect_field_name,
                     nextval,
                 )
             )
-    return ''
+    return u''
 
 
 def get_paginator(request, qs):
@@ -159,3 +156,29 @@ def check_allow_for_uri(uri):
             return False
 
     return True
+
+
+try:
+    from django.utils.duration import duration_string
+except ImportError:
+    # Django < 1.8
+    def duration_string(duration):
+        ''' Taken straight from Django 1.8 - django/utils/duration.py
+        '''
+        days = duration.days
+        seconds = duration.seconds
+        microseconds = duration.microseconds
+
+        minutes = seconds // 60
+        seconds = seconds % 60
+
+        hours = minutes // 60
+        minutes = minutes % 60
+
+        string = '{:02d}:{:02d}:{:02d}'.format(hours, minutes, seconds)
+        if days:
+            string = '{} '.format(days) + string
+        if microseconds:
+            string += '.{:06d}'.format(microseconds)
+
+        return string
