@@ -1,7 +1,13 @@
+# -*- coding: utf-8 -*-
 from .helpers import check_allow_for_user, check_allow_for_uri, User
 
+try:
+    from django.utils.deprecation import MiddlewareMixin
+except ImportError:
+    MiddlewareMixin = object  # fallback for Django < 1.10
 
-class ImpersonateMiddleware(object):
+
+class ImpersonateMiddleware(MiddlewareMixin):
     def process_request(self, request):
         request.user.is_impersonate = False
         request.impersonator = None
@@ -11,10 +17,10 @@ class ImpersonateMiddleware(object):
             new_user_id = request.session['_impersonate']
             if isinstance(new_user_id, User):
                 # Edge case for issue 15
-                new_user_id = new_user_id.id
+                new_user_id = new_user_id.pk
 
             try:
-                new_user = User.objects.get(id=new_user_id)
+                new_user = User.objects.get(pk=new_user_id)
             except User.DoesNotExist:
                 return
 
