@@ -1,12 +1,17 @@
-from django.conf import settings
+# -*- coding: utf-8 -*-
+import logging
 from django.db.models import Q
+from django.conf import settings
 from django.shortcuts import get_object_or_404, redirect, render
+
 from .decorators import allowed_user_required
+from .signals import session_begin, session_end
 from .helpers import (
     get_redir_path, get_redir_arg, get_paginator, get_redir_field,
     check_allow_for_user, users_impersonable, User
 )
-from .signals import session_begin, session_end
+
+logger = logging.getLogger(__name__)
 
 
 @allowed_user_required
@@ -17,10 +22,18 @@ def impersonate(request, uid):
 
         The middleware will then pick up on it and adjust the
         request object as needed.
+
+        Also store the user's 'starting'/'original' URL so
+        we can return them to it.
     '''
     new_user = get_object_or_404(User, pk=uid)
     if check_allow_for_user(request, new_user):
-        request.session['_impersonate'] = new_user.id
+        request.session['_impersonate'] = new_user.pk
+        prev_path = request.META.get('HTTP_REFERER')
+        if prev_path:
+            request.session['_impersonate_prev_path'] = \
+                                request.build_absolute_uri(prev_path)
+
         request.session.modified = True  # Let's make sure...
         # can be used to hook up auditing of the session
         session_begin.send(
@@ -34,8 +47,24 @@ def impersonate(request, uid):
 
 def stop_impersonate(request):
     ''' Remove the impersonation object from the session
+        and ideally return the user to the original path
+        they were on.
     '''
     impersonating = request.session.pop('_impersonate', None)
+    if impersonating is not None:
+        try:
+            impersonating = User.objects.get(pk=impersonating)
+        except User.DoesNotExist:
+            # Should never be reached.
+            logger.info(
+                (u'NOTICE: User being impersonated (PK '
+                 u'{0}) no longer exists.').format(impersonating)
+            )
+            impersonating = None
+
+    original_path = request.session.pop('_impersonate_prev_path', None)
+    use_refer = getattr(settings, 'IMPERSONATE_USE_HTTP_REFERER', False)
+
     if impersonating is not None:
         request.session.modified = True
         session_end.send(
@@ -44,7 +73,10 @@ def stop_impersonate(request):
             impersonating=impersonating,
             request=request
         )
-    return redirect(get_redir_path(request))
+
+    dest = original_path \
+        if original_path and use_refer else get_redir_path(request)
+    return redirect(dest)
 
 
 @allowed_user_required
@@ -84,7 +116,7 @@ def search_users(request, template):
           * redirect_field - hidden input field with redirect argument,
                               put this inside search form
     '''
-    query = request.GET.get('q', '')
+    query = request.GET.get('q', u'')
 
     # get username field
     username_field = getattr(User, 'USERNAME_FIELD', 'username')
